exports.module = function (config, errback) {
   var exports = {},
       activeStopsTimer,
       routeConfigTimer,
       log = config.log,
       nextbus = require('../models/nextbusjs').client,
       async = require('async'),
       myconfig = config.modules.nextbus,
       writeFile = require('fs').writeFile,
       rename = require('fs').rename,
       agencyCache,
       nwk,
       statusData = {},
       nb;

   function status (callback) {
      callback(null, statusData);
   }

   function doActiveStops () {
      if (!doActiveStops.working) {
         doActiveStops.working = true;
      } else {
         return;
      }

      async.parallel({
         nwk: function (callback) {
            nwk.guessActive(callback);
         },
         nb: function (callback) {
            nb.guessActive(callback);
         }
      }, function (err, results) {

         if (err) {
            log.notice(JSON.stringify(err, false, 2));
            return;
         }

         // write the output
         async.parallel([
            function (callback) {
               writeFile(
                  config.paths.temp + 'nbactivestops.txt', 
                  JSON.stringify(results.nb),
                  callback
               );
            },
            function (callback) {
               writeFile(
                  config.paths.temp + 'nwkactivestops.txt', 
                  JSON.stringify(results.nwk),
                  callback
               );
            }
         ], function (err) {
            if (err) {
               log.notice(JSON.stringify(err, false, 2));
               return;
            }
            // now move the files to their final resting place

            async.parallel([
               function (callback) {
                  rename(
                     config.paths.temp + 'nbactivestops.txt',
                     config.paths.output + 'nbactivestops.txt',
                     callback
                  );
               },
               function (callback) {
                  rename(
                     config.paths.temp + 'nwkactivestops.txt',
                     config.paths.output + 'nwkactivestops.txt',
                     callback
                  );
               }
            ], function (err) {
               if (err) {
                  log.notice(JSON.stringify(err, false, 2));
                  return;
               }
               
               // finally done
               doActiveStops.working = false;
               statusData.active = {
                  last: new Date()
               };
               log.info("bus: activestops written");
            });
         });
      });

   }

   function doRouteConfig (done) {
      // set this to true so that doActiveStops doesn't run while we're
      // trying to refresh the route config

      doActiveStops.working = true;
      async.parallel([
         function (callback) {
            nb.cacheAgency('rutgers', callback);
         },
         function (callback) {
            nwk.cacheAgency('rutgers-newark', callback);
         }
      ], function (err, results) {
         if (err) {
            throw err;
         }
         agencyCache = results;
         doActiveStops.working = false;
         log.info('bus: got agency cache, writing to disk');

         // write the output
         async.parallel([
            function (callback) {
               writeFile(
                  config.paths.temp + 'rutgersrouteconfig.txt', 
                  JSON.stringify(results[0]),
                  callback
               );
            },
            function (callback) {
               writeFile(
                  config.paths.temp + 'rutgers-newarkrouteconfig.txt', 
                  JSON.stringify(results[1]),
                  callback
               );
            }
         ], function (err) {
            if (err) {
               log.notice(JSON.stringify(err, false, 2));
               return;
            }
            // now move the files to their final resting place

            async.parallel([
               function (callback) {
                  rename(
                     config.paths.temp + 'rutgersrouteconfig.txt', 
                     config.paths.output + 'rutgersrouteconfig.txt', 
                     callback
                  );
               },
               function (callback) {
                  rename(
                     config.paths.temp + 'rutgers-newarkrouteconfig.txt', 
                     config.paths.output + 'rutgers-newarkrouteconfig.txt', 
                     callback
                  );
               }
            ], function (err) {
               if (err) {
                  log.error(JSON.stringify(err, false, 2));
                  return;
               }
               
               // finally done
               log.info("bus: route config written");

               statusData.routeConfig = {
                  last: new Date()
               };

               if (done) {
                  done();
               }
            });
         });
      });
            
   }

   function start () {
      // setup the nwk and nb nextbus clients
      nwk = nextbus();
      nb = nextbus();

      doRouteConfig(function () {
         log.info('bus: got agency info, starting background tasks');
         doActiveStops();
         activeStopsTimer = setInterval(
            doActiveStops, Number(myconfig.intervals.running) * 1000 * 60
         );
         routeConfigTimer = setInterval(
            doRouteConfig, Number(myconfig.intervals.routeconfig) * 1000 * 60
         );
      });
   }

   function shutdown (done) {
      clearInterval(activeStopsTimer);
      clearInterval(routeConfigTimer);
      done();
   }

   exports.start = start;
   exports.shutdown = shutdown;
   exports.status = status;

   return exports;
};
