var assert       = require('assert');
var syslogParser = require('../lib/glossy/parse.js');

assert.ok(syslogParser, 'parser loaded');

//TODO: Add more edge cases and other shenanigans here
var messages = [
	"<165>1 2003-08-24T05:14:15.000003-07:00 192.0.2.1 myproc 8710 - - %% It's time to make the do-nuts.",
	'<165>1 2003-10-11T22:14:15.003Z mymachine.example.com evntslog - ID47 [exampleSDID@32473 iut="3" eventSource= "Application" eventID="1011"] BOMAn application event log entry...',
	'<165>1 2003-10-11T22:14:15.003Z mymachine.example.com evntslog - ID47 [exampleSDID@32473 iut="3" eventSource= "Application" eventID="1011"][examplePriority@32473 class="high"] ',
	"<34>1 2003-10-11T22:14:15.003Z mymachine.example.com su - ID47 - BOM'su root' failed for lonvick on /dev/pts/8",
	"<13>Feb  5 17:32:18 10.0.0.99 Use the BFG!",
	"<34>Oct 11 22:14:15 mymachine su: 'su root' failed for lonvick on /dev/pts/8",
    '<110>1 2009-05-03T14:00:39.519307+02:00 host.example.org syslogd 2138 - [ssign-cert VER="0111" RSID="1" SG="0" SPRI="0" TPBL="587" INDEX="1" FLEN="587" FRAG="2009-05-03T14:00:39.519005+02:00 K BACsLMZ NCV2NUAwe4RAeAnSQuvv2KS51SnHFAaWJNU2XVDYvW1LjmJgg4vKvQPo3HEOD+2hEkt1zcXADe03u5pmHoWy5FGiyCbglYxJkUJJrQqlTSS6vID9yhsmEnh07w3pOsxmb4qYo0uWQrAAenBweVMlBgV3ZA5IMA8xq8l+i8wCgkWJjCjfLar7s+0X3HVrRroyARv8EAIYoxofh9m N8n821BTTuQnz5hp40d6Z3UudKePu2di5Mx3GFelwnV0Qh5mSs0YkuHJg0mcXyUAoeYry5X6482fUxbm+gOHVmYSDtBmZEB8PTEt8Os8aedWgKEt/E4dT+Hmod4omECLteLXxtScTMgDXyC+bSBMjRRCaeWhHrYYdYBACCWMdTc12hRLJTn8LX99kv1I7qwgieyna8GCJv/rEgC ssS9E1qARM+h19KovIUOhl4VzBw3rK7v8Dlw/CJyYDd5kwSvCwjhO21LiReeS90VPYuZFRC1B82Sub152zOqIcAWsgd4myCCiZbWBsuJ8P0gtarFIpleNacCc6OV3i2Rg==" SIGN="AKAQEUiQptgpd0lKcXbuggGXH/dCdQCgdysrTBLUlbeGAQ4vwrnLOqSL7+c="]',
    '<110>1 2009-05-03T14:00:39.529966+02:00 host.example.org syslogd 2138 - [ssign VER="0111" RSID="1" SG="0" SPRI="0" GBC="2" FMN="1" CNT="7" HB="K6wzcombEvKJ+UTMcn9bPryAeaU= zrkDcIeaDluypaPCY8WWzwHpPok= zgrWOdpx16ADc7UmckyIFY53icE= XfopJ+S8/hODapiBBCgVQaLqBKg= J67gKMFl/OauTC20ibbydwIlJC8= M5GziVgB6KPY3ERU1HXdSi2vtdw= Wxd/lU7uG/ipEYT9xeqnsfohyH0=" SIGN="AKBbX4J7QkrwuwdbV7Taujk2lvOf8gCgC62We1QYfnrNHz7FzAvdySuMyfM="]'
];

for(message in messages) {
	parsed = syslogParser.parse(messages[message]);
	assert.ok(parsed);
};

var firstParsed = syslogParser.parse(messages[0]);
assert.ok(firstParsed, 'First message parsed successfully');
assert.equal(firstParsed.facility, 'local4', 'Facility value matches');
assert.equal(firstParsed.severity, 'notice', 'Severity value matches');

syslogParser.parse(messages[1], function(secondParsed){
	assert.ok(secondParsed, 'Second message parsed, assert using callback');
	assert.equal(secondParsed.host, 'mymachine.example.com', 'hostname matches');
});

// These values match the hasing algorithm values as defined in RFC 5848,
// Section 4.2.1.
var signedBlockValues = {
    hashAlgorithm: [
        null,
        'SHA1',
        'SHA256'
    ]
};

console.log( parseSignedBlock(syslogParser.parse(messages[7])) );

/*
 *  Make sense of signed block messages
 *  @param {Object} block the parsed structured data containing signed data
 *  @return {Object} validatedBlock translated and named values, binary
 *      elements will be Buffer objects, if available
 */
function parseSignedBlock(block) {
    if(typeof block != 'object') return false;

    var signedBlock    = { };
    var validatedBlock = { };
    // Figure out where in the object the keys live...
    if(block.structuredData && block.structuredData.ssign) {
        signedBlock = block.structuredData.ssign;
    } else if(block.ssign) {
        signedBlock = block.ssign
    } else if(block.VER) {
        signedBlock = block;
    } else {
        return false;
    }

    var versionMatch = signedBlock.VER.match(/^(\d{2})(\d|\w)(\d)$/);
    if(versionMatch != null) {
        validatedBlock.version        = versionMatch[1];
        validatedBlock.hashAlgorithm  = parseInt(versionMatch[2]);
        validatedBlock.hashAlgoString = signedBlockValues.hashAlgorithm[validatedBlock.hashAlgorithm];
        validatedBlock.sigScheme      = parseInt(versionMatch[3]);
    }

    validatedBlock.rebootSessionID   = parseInt(signedBlock.RSID);
    validatedBlock.signatureGroup    = parseInt(signedBlock.SG);
    validatedBlock.signaturePriority = parseInt(signedBlock.SPRI);
    validatedBlock.globalBlockCount  = parseInt(signedBlock.GBC);
    validatedBlock.firstMsgNumber    = parseInt(signedBlock.FMN);
    validatedBlock.msgCount          = parseInt(signedBlock.CNT);
    validatedBlock.hashBlock         = signedBlock.HB.split(/\s/);

    // Check to see if we're in node or have a Buffer type
    if(typeof Buffer == 'function') {
        for(hash in validatedBlock.hashBlock) {
            validatedBlock.hashBlock[hash] = new Buffer(
                validatedBlock.hashBlock[hash], encoding='base64'); 
        }
        validatedBlock.thisSignature = new Buffer(
            signedBlock.SIGN, encoding='base64');
    } else {
        validatedBlock.thisSignature = signedBlock.SIGN;
    }

    return validatedBlock;
    
}


