exports.main = function (command, options) {
   var dnode      = require('dnode'),        // rpc framework
       log        = require('winston'),      // logger
       yaml       = require('js-yaml'),
       _          = require('underscore'),
       version    = require('./revision').ver,
       async      = require('async'),
       handlers   = {},
       modules    = {},
       error      = false,
       config,
       rpc;

   require('winston-syslog').Syslog;

   handlers.status = function (done) {
      if (error) {
         done(error, null);
      } else {
         async.map(_.keys(modules), function (item, callback) {
            modules[item].status(callback);
         }, function (err, data) {
            done(null, data);
         });
      }
   };

   handlers.reseterror = function (done) {
      if (error) {
         error = false;
         done(null, 'removed error condition');
      } else {
         done('error condition is not set', null);
      }
      //TODO: winston
   };

   handlers.stop = function (done) {
      shutdown(function () {
         process.exit(0);
      });
   };

   function onError (name, error) {
      //TODO: winston
   }

   function clearRequireCache () {
      for (var i in require.cache) {
         if (require.cache.hasOwnProperty(i)) {
            delete require.cache[i];
         }
      }
   }

   function shutdown (done) {
      async.forEach(_.keys(modules), function (item, next) {
         modules[item].shutdown(next);
      }, function (err) {
         if (err) {
            log.error(err);
         } else {
            done();
         }
      });
      log.info('unloaded modules');
   }

   function initialize () {
      // TODO: fix paths
      clearRequireCache();
      config = require('./config.yaml').shift();
      log.setLevels(log.config.syslog.levels);
      log.add(log.transports.Syslog, config.syslog);
      config.log = log;
      _.each(config.modules, function (cfg, module) { 
         modules[module] = require('./modules/' + module).module(config, onError);
         modules[module].start();
      });
      log.info('loaded config and modules');
   }

   process.on('SIGHUP', function () {
      //TODO: winston
      log.log('received SIGHUP, reloading config and modules');
      shutdown(initialize);
   });

   process.on('SIGKILL', function () {
      log.log('received SIGKILL, shutting down');
      shutdown(function () {
         process.exit(0);
      });
   });

   initialize();

   if (!options.debug) {
      log.remove(log.transports.Console);
   }

   process.title = "RU Mobile " + version.ver + "." + version.rev;

   rpc = dnode(handlers).listen(63200);
};

